@@ -4,8 +4,6 @@
 
 #include "ui/views/accessible_pane_view.h"
 
-#include "base/memory/ptr_util.h"
-#include "base/message_loop/message_loop.h"
 #include "ui/accessibility/ax_node_data.h"
 #include "ui/views/focus/focus_search.h"
 #include "ui/views/view_tracker.h"
@@ -26,8 +24,9 @@ class AccessiblePaneViewFocusSearch : public FocusSearch {
 
  protected:
   View* GetParent(View* v) override {
-    return accessible_pane_view_->ContainsForFocusSearch(root(), v) ?
-        accessible_pane_view_->GetParentForFocusSearch(v) : NULL;
+    return accessible_pane_view_->ContainsForFocusSearch(root(), v)
+               ? accessible_pane_view_->GetParentForFocusSearch(v)
+               : nullptr;
   }
 
   // Returns true if |v| is contained within the hierarchy rooted at |root|.
@@ -44,7 +43,7 @@ class AccessiblePaneViewFocusSearch : public FocusSearch {
 AccessiblePaneView::AccessiblePaneView()
     : pane_has_focus_(false),
       allow_deactivate_on_esc_(false),
-      focus_manager_(NULL),
+      focus_manager_(nullptr),
       home_key_(ui::VKEY_HOME, ui::EF_NONE),
       end_key_(ui::VKEY_END, ui::EF_NONE),
       escape_key_(ui::VKEY_ESCAPE, ui::EF_NONE),
@@ -110,7 +109,7 @@ bool AccessiblePaneView::SetPaneFocusAndFocusDefault() {
 }
 
 views::View* AccessiblePaneView::GetDefaultFocusableChild() {
-  return NULL;
+  return nullptr;
 }
 
 View* AccessiblePaneView::GetParentForFocusSearch(View* v) {
@@ -136,7 +135,10 @@ views::View* AccessiblePaneView::GetFirstFocusableChild() {
   FocusTraversable* dummy_focus_traversable;
   views::View* dummy_focus_traversable_view;
   return focus_search_->FindNextFocusableView(
-      NULL, false, views::FocusSearch::DOWN, false,
+      nullptr, FocusSearch::SearchDirection::kForwards,
+      FocusSearch::TraversalDirection::kDown,
+      FocusSearch::StartingViewPolicy::kSkipStartingView,
+      FocusSearch::AnchoredDialogPolicy::kCanGoIntoAnchoredDialog,
       &dummy_focus_traversable, &dummy_focus_traversable_view);
 }
 
@@ -144,7 +146,10 @@ views::View* AccessiblePaneView::GetLastFocusableChild() {
   FocusTraversable* dummy_focus_traversable;
   views::View* dummy_focus_traversable_view;
   return focus_search_->FindNextFocusableView(
-      this, true, views::FocusSearch::DOWN, false,
+      this, FocusSearch::SearchDirection::kBackwards,
+      FocusSearch::TraversalDirection::kDown,
+      FocusSearch::StartingViewPolicy::kSkipStartingView,
+      FocusSearch::AnchoredDialogPolicy::kCanGoIntoAnchoredDialog,
       &dummy_focus_traversable, &dummy_focus_traversable_view);
 }
 
@@ -155,7 +160,7 @@ views::FocusTraversable* AccessiblePaneView::GetPaneFocusTraversable() {
   if (pane_has_focus_)
     return this;
   else
-    return NULL;
+    return nullptr;
 }
 
 bool AccessiblePaneView::AcceleratorPressed(
@@ -205,7 +210,7 @@ void AccessiblePaneView::SetVisible(bool flag) {
 }
 
 void AccessiblePaneView::GetAccessibleNodeData(ui::AXNodeData* node_data) {
-  node_data->role = ui::AX_ROLE_PANE;
+  node_data->role = ax::mojom::Role::kPane;
 }
 
 void AccessiblePaneView::RequestFocus() {
@@ -248,12 +253,12 @@ views::FocusSearch* AccessiblePaneView::GetFocusSearch() {
 
 views::FocusTraversable* AccessiblePaneView::GetFocusTraversableParent() {
   DCHECK(pane_has_focus_);
-  return NULL;
+  return nullptr;
 }
 
 views::View* AccessiblePaneView::GetFocusTraversableParentView() {
   DCHECK(pane_has_focus_);
-  return NULL;
+  return nullptr;
 }
 
 }  // namespace views
