@@ -185,10 +185,8 @@ void QQuickWebEngineDownloadItemPrivate::update(const ProfileAdapterClient::Down
         Q_EMIT q->totalBytesChanged();
     }
 
-    if (info.done != downloadFinished) {
-        downloadFinished = info.done;
-        Q_EMIT q->isFinishedChanged();
-    }
+    if (info.done)
+        setFinished();
 
     if (info.paused != downloadPaused) {
         downloadPaused = info.paused;
@@ -206,6 +204,17 @@ void QQuickWebEngineDownloadItemPrivate::updateState(QQuickWebEngineDownloadItem
     }
 }
 
+void QQuickWebEngineDownloadItemPrivate::setFinished()
+{
+    Q_Q(QQuickWebEngineDownloadItem);
+
+    if (downloadFinished)
+        return;
+
+    downloadFinished = true;
+    Q_EMIT q->isFinishedChanged();
+}
+
 /*!
     \qmlmethod void WebEngineDownloadItem::accept()
 
@@ -255,6 +264,7 @@ void QQuickWebEngineDownloadItem::cancel()
         return;
 
     d->updateState(QQuickWebEngineDownloadItem::DownloadCancelled);
+    d->setFinished();
 
     // We directly cancel the download if the user cancels before
     // it even started, so no need to notify the profile here.
@@ -593,12 +603,14 @@ bool QQuickWebEngineDownloadItem::isPaused() const
 }
 
 /*!
-    \qmlproperty bool WebEngineDownloadItem::view
+    \qmlproperty WebEngineView WebEngineDownloadItem::view
     \readonly
     \since QtWebEngine 1.8
 
     Returns the view the download was requested on. If the download was not triggered by content in a view,
     \c nullptr is returned.
+
+    \sa WebEngineView
 */
 QQuickWebEngineView *QQuickWebEngineDownloadItem::view() const
 {
@@ -615,6 +627,8 @@ QQuickWebEngineDownloadItem::QQuickWebEngineDownloadItem(QQuickWebEngineDownload
 
 QQuickWebEngineDownloadItem::~QQuickWebEngineDownloadItem()
 {
+    if (!isFinished())
+        cancel();
 }
 
 QT_END_NAMESPACE
